@@ -1,24 +1,45 @@
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD.
+# See https://nomad-lab.eu for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 import os
 import numpy as np
 import pint
 import logging
 
 import panedr
-import MDAnalysis
+try:
+    import MDAnalysis
+except Exception:
+    logging.warn('Required module MDAnalysis not found.')
+    MDAnalysis = False
 
 from .metainfo import m_env
 from nomad.parsing.parser import FairdiParser
 
-from nomad.parsing.file_parser import UnstructuredTextFileParser, Quantity, FileParser
-from nomad.datamodel.metainfo.public import section_run, section_sampling_method,\
-    section_system, section_energy_contribution, section_single_configuration_calculation
-from nomad.datamodel.metainfo.common import section_topology, section_interaction
+from nomad.parsing.file_parser import TextParser, Quantity, FileParser
+from nomad.datamodel.metainfo.common_dft import Run, SamplingMethod, System,\
+    EnergyContribution, SingleConfigurationCalculation, Topology, Interaction
 from .metainfo.gromacs import x_gromacs_section_control_parameters, x_gromacs_section_input_output_files
 
 MOL = 6.022140857e+23
 
 
-class GromacsLogParser(UnstructuredTextFileParser):
+class GromacsLogParser(TextParser):
     def __init__(self):
         run_control = [
             'integrator', 'tinit', 'dt', 'nsteps', 'init-step', 'simulation-part',
@@ -144,7 +165,7 @@ class GromacsLogParser(UnstructuredTextFileParser):
             'density-guided-simulation-normalize-densities', 'density-guided-simulation-adaptive-force-scaling',
             'density-guided-simulation-adaptive-force-scaling-time-constant']
 
-        self._commands = ['GROMACS version']
+        self._commands = ['version']
 
         self._commands +=\
             run_control + langevin_dynamics + energy_minimization + shell_molecular_dynamics +\
@@ -170,15 +191,15 @@ class GromacsLogParser(UnstructuredTextFileParser):
             self._quantities.append(
                 Quantity(
                     command, r'\s*%s\s*\[*\s*\d*\s*\]*\s*[=:]+([\s\S]*?\n)' % command,
-                    str_operation=str_op)
+                    str_operation=str_op, repeats=False)
             )
 
     def get_pbc(self):
-        pbc = self.get('pbc', ['xyz'])[0]
+        pbc = self.get('pbc', 'xyz')
         return ['x' in pbc, 'y' in pbc, 'z' in pbc]
 
     def get_sampling_settings(self):
-        integrator = self.get('integrator', ['md'])[0].lower()
+        integrator = self.get('integrator', 'md').lower()
         if integrator in ['l-bfgs', 'cg', 'steep']:
             sampling_method = 'geometry_optimization'
         elif integrator in ['bd']:
@@ -187,10 +208,10 @@ class GromacsLogParser(UnstructuredTextFileParser):
             sampling_method = 'molecular_dynamics'
 
         ensemble_type = 'NVE' if sampling_method == 'molecular_dynamics' else None
-        tcoupl = self.get('tcoupl', ['no'])[0].lower()
+        tcoupl = self.get('tcoupl', 'no').lower()
         if tcoupl != 'no':
             ensemble_type = 'NVT'
-            pcoupl = self.get('pcoupl', ['no'])[0].lower()
+            pcoupl = self.get('pcoupl', 'no').lower()
             if pcoupl != 'no':
                 ensemble_type = 'NPT'
 
@@ -199,30 +220,30 @@ class GromacsLogParser(UnstructuredTextFileParser):
             ensemble_type=ensemble_type)
 
     def get_tpstat_settings(self):
-        target_T = self.get('ref-t', [0], unit='K')
+        target_T = self.get('ref-t', 0, unit='K')
 
         thermostat_type = None
-        tcoupl = self.get('tcoupl', ['no'])[0].lower()
+        tcoupl = self.get('tcoupl', 'no').lower()
         if tcoupl != 'no':
             thermostat_type = 'Velocity Rescaling' if tcoupl == 'v-rescale' else tcoupl.title()
 
-        thermostat_tau = self.get('tau-t', [0], unit='ps')
+        thermostat_tau = self.get('tau-t', 0, unit='ps')
 
         # TODO infer langevin_gamma [s] from bd_fric
-        # bd_fric = self.get('bd-fric', [0], unit='amu/ps')
+        # bd_fric = self.get('bd-fric', 0, unit='amu/ps')
         langevin_gamma = None
 
-        target_P = self.get('ref-p', [0], unit='bar')
+        target_P = self.get('ref-p', 0, unit='bar')
         # if P is array e.g. for non-isotropic pressures, get average since metainfo is float
         if hasattr(target_P, 'shape'):
             target_P = np.average(target_P)
 
         barostat_type = None
-        pcoupl = self.get('pcoupl', ['no'])[0].lower()
+        pcoupl = self.get('pcoupl', 'no').lower()
         if pcoupl != 'no':
             barostat_type = pcoupl.title()
 
-        barostat_tau = self.get('tau-p', [0], unit='ps')
+        barostat_tau = self.get('tau-p', 0, unit='ps')
 
         return dict(
             target_T=target_T, thermostat_type=thermostat_type, thermostat_tau=thermostat_tau,
@@ -336,6 +357,9 @@ class MDAnalysisParser(FileParser):
         return self._file_handler
 
     def parse(self, key):
+        if not MDAnalysis:
+            return
+
         if self._results is None:
             self._results = dict()
 
@@ -386,7 +410,6 @@ class GromacsParser(FairdiParser):
             name='parsers/gromacs', code_name='Gromacs', code_homepage='http://www.gromacs.org/',
             domain='dft', mainfile_contents_re=r'gmx mdrun, VERSION')
         self._metainfo_env = m_env
-
         self.log_parser = GromacsLogParser()
         self.traj_parser = MDAnalysisParser()
         self.energy_parser = GromacsEDRParser()
@@ -433,25 +456,26 @@ class GromacsParser(FairdiParser):
         if sec_sccs:
             if len(sec_sccs) != n_evaluations:
                 self.logger.warn(
-                    '''Mismatch in number of calculations %d and number of thermodynamic
-                    evaluations %d, will create new sections''' % (len(sec_sccs), n_evaluations))
+                    '''Mismatch in number of calculations and number of thermodynamic
+                    evaluations, will create new sections''',
+                    data=dict(n_calculations=len(sec_sccs), n_evaluations=n_evaluations))
 
             else:
                 create_scc = False
 
-        timestep = self.traj_parser.get('timestep', unit='ps')
+        timestep = self.traj_parser.get('timestep', 1.0, unit='ps')
 
         keys = self.energy_parser.get_keys()
         for n in range(n_evaluations):
             if create_scc:
-                sec_scc = sec_run.m_create(section_single_configuration_calculation)
+                sec_scc = sec_run.m_create(SingleConfigurationCalculation)
             else:
                 sec_scc = sec_sccs[n]
 
             for key in keys:
                 val = self.energy_parser.get(key)[n]
                 if key in energy_keys_mapping:
-                    sec_energy = sec_scc.m_create(section_energy_contribution)
+                    sec_energy = sec_scc.m_create(EnergyContribution)
                     sec_energy.energy_contibution_kind = energy_keys_mapping[key]
                     sec_energy.energy_contribution_value = val
 
@@ -470,7 +494,7 @@ class GromacsParser(FairdiParser):
     def parse_topology(self):
         sec_run = self.archive.section_run[-1]
 
-        sec_topology = sec_run.m_create(section_topology)
+        sec_topology = sec_run.m_create(Topology)
         try:
             n_atoms = self.traj_parser.get('n_atoms', [0])[0]
         except Exception:
@@ -484,7 +508,7 @@ class GromacsParser(FairdiParser):
         for interaction in interactions:
             if not interaction[0] or not interaction[1]:
                 continue
-            sec_interaction = sec_topology.m_create(section_interaction)
+            sec_interaction = sec_topology.m_create(Interaction)
             sec_interaction.interaction_kind = interaction[0]
             sec_interaction.interaction_parameters = interaction[1]
 
@@ -498,8 +522,9 @@ class GromacsParser(FairdiParser):
         if sec_sccs:
             if len(sec_sccs) != n_frames:
                 self.logger.warn(
-                    '''Mismatch in number of calculations %d and number of systems %d,
-                    will create new sections''' % (len(sec_sccs), n_frames))
+                    '''Mismatch in number of calculations and number of structures,
+                    will create new sections''',
+                    data=dict(n_calculations=len(sec_sccs), n_structures=n_frames))
 
             else:
                 create_scc = False
@@ -510,7 +535,7 @@ class GromacsParser(FairdiParser):
             if positions is None:
                 continue
 
-            sec_system = sec_run.m_create(section_system)
+            sec_system = sec_run.m_create(System)
             sec_system.number_of_atoms = self.traj_parser.get_n_atoms(n)
             sec_system.configuration_periodic_dimensions = pbc
             sec_system.simulation_cell = self.traj_parser.get_cell(n)
@@ -524,14 +549,14 @@ class GromacsParser(FairdiParser):
             forces = self.traj_parser.get_forces(n)
             if forces is not None:
                 if create_scc:
-                    sec_scc = sec_run.m_create(section_single_configuration_calculation)
+                    sec_scc = sec_run.m_create(SingleConfigurationCalculation)
                 else:
                     sec_scc = sec_sccs[n]
                 sec_scc.atom_forces = forces
 
     def parse_sampling_method(self):
         sec_run = self.archive.section_run[-1]
-        sec_sampling_method = sec_run.m_create(section_sampling_method)
+        sec_sampling_method = sec_run.m_create(SamplingMethod)
 
         sampling_settings = self.log_parser.get_sampling_settings()
 
@@ -539,10 +564,10 @@ class GromacsParser(FairdiParser):
         sec_sampling_method.ensemble_type = sampling_settings['ensemble_type']
         sec_sampling_method.x_gromacs_integrator_type = sampling_settings['integrator_type']
 
-        timestep = self.log_parser.get('dt', [0])[0]
+        timestep = self.log_parser.get('dt', 0)
         sec_sampling_method.x_gromacs_integrator_dt = timestep
 
-        nsteps = self.log_parser.get('nsteps', [0])[0]
+        nsteps = self.log_parser.get('nsteps', 0)
         sec_sampling_method.x_gromacs_number_of_steps_requested = nsteps
 
         tp_settings = self.log_parser.get_tpstat_settings()
@@ -582,19 +607,22 @@ class GromacsParser(FairdiParser):
         sec_control_parameters = sec_run.m_create(x_gromacs_section_control_parameters)
         keys = self.log_parser.keys()
         for key in keys:
-            val = self.log_parser.get(key, [None])[0]
+            val = self.log_parser.get(key)
             if val is None:
                 continue
             key = 'x_gromacs_inout_control_%s' % key.replace('-', '').lower()
             if hasattr(sec_control_parameters, key):
                 setattr(sec_control_parameters, key, str(val))
 
-    def _init_parsers(self):
+    def init_parser(self):
         self.log_parser.mainfile = self.filepath
         self.log_parser.logger = self.logger
         self.traj_parser.logger = self.logger
         self.energy_parser.logger = self.logger
 
+    def reuse_parser(self, parser):
+        self.log_parser.quantities = parser.log_parser.quantities
+
     def parse(self, filepath, archive, logger):
         self.filepath = os.path.abspath(filepath)
         self.archive = archive
@@ -603,11 +631,11 @@ class GromacsParser(FairdiParser):
         self._gromacs_files = os.listdir(self._maindir)
         self._basename = os.path.basename(filepath).split('.')[0]
 
-        self._init_parsers()
+        self.init_parser()
 
-        sec_run = self.archive.m_create(section_run)
+        sec_run = self.archive.m_create(Run)
 
-        version = self.log_parser.get('GROMACS version', [['VERSION', 'unknown']])[0]
+        version = self.log_parser.get('version', ['VERSION', 'unknown'])
         sec_run.program_name = 'GROMACS'
         sec_run.program_version = version[1]
 
