@@ -134,6 +134,8 @@ class PhononNormalizer(Normalizer):
 
     def _get_n_imaginary_frequencies(self):
         scc = self.section_run.section_single_configuration_calculation
+        if not scc:
+            return
         sec_band = scc[0].section_k_band
         result = 0
         for band_segment in sec_band[0].section_k_band_segment:
@@ -303,7 +305,11 @@ class WorkflowNormalizer(Normalizer):
         self._phonon_programs = ['phonopy']
 
     def _resolve_workflow_type_vasp(self):
-        ibrion = self.section_run.section_method[0].x_vasp_incarOut_IBRION
+        try:
+            ibrion = self.section_run.section_method[0].x_vasp_incarOut_IBRION
+        except Exception:
+            ibrion = 1
+
         if ibrion == 0:
             workflow_type = "molecular_dynamics"
         else:
