@@ -232,9 +232,10 @@ class DataTextParser(FileParser):
     def __init__(self, **kwargs):
         self._dtype: Type = kwargs.get('dtype', float)
         mainfile: str = kwargs.get('mainfile', None)
+        self._mainfile_contents: str = kwargs.get('mainfile_contents', None)
         logger = kwargs.get('logger', None)
         logger = logger if logger is not None else logging
-        super().__init__(mainfile, logger=logger)
+        super().__init__(mainfile, logger=logger, open=kwargs.get('open', None))
         self.init_parameters()
 
     def init_parameters(self):
@@ -249,11 +250,13 @@ class DataTextParser(FileParser):
         Returns the loaded data
         '''
         if self._file_handler is None:
-            if self.mainfile is None:
-                return
-
             try:
-                self._file_handler = np.loadtxt(self.mainfile, dtype=self._dtype)
+                if self.mainfile is not None:
+                    self._file_handler = np.loadtxt(self.mainfile)
+                else:
+                    if self._mainfile_contents is None:
+                        self._mainfile_contents = self.mainfile_obj.read()
+                    self._file_handler = np.frombuffer(self._mainfile_contents, dtype=self._dtype)
             except Exception:
                 return
 
@@ -278,7 +281,7 @@ class TextParser(FileParser):
         file_length: length of the chunk to be read from the file
     '''
     def __init__(self, mainfile=None, quantities=None, logger=None, findall=True, **kwargs):
-        super().__init__(mainfile, logger)
+        super().__init__(mainfile, logger=logger, open=kwargs.get('open', None))
         self._quantities: List[Quantity] = quantities
         self.findall: bool = findall
         self._kwargs = kwargs
@@ -287,6 +290,20 @@ class TextParser(FileParser):
         self._file_pad: int = 0
         if quantities is None:
             self.init_quantities()
+        # check quantity patterns are valid
+        re_has_group = re.compile(r'\(.+\)')
+        for i in range(len(self.quantities) - 1, -1, -1):
+            valid = True
+            try:
+                if re_has_group.search(self.quantities[i].re_pattern.pattern.decode()) is None:
+                    valid = False
+            except Exception:
+                valid = False
+            if not valid:
+                self.logger.error(
+                    'Invalid quantity pattern', data=dict(quantity=self.quantities[i].name))
+                self.quantities.pop(i)
+        self._re_findall = None
 
     def copy(self):
         '''
@@ -364,11 +381,17 @@ class TextParser(FileParser):
             yield key, self.get(key)
 
     def _parse_quantities(self, quantities):
-        re_findall = '|'.join([q.re_pattern.pattern.decode() for q in quantities])
-        if len(quantities) == 1:
-            # necessary to add a dummy variable to make multiple matches
-            re_findall = '%s|(__dummy__)' % re_findall
-        re_findall = re_findall.encode()
+        if len(self._results) == 0 and self._re_findall is not None:
+            # maybe an opt
+            re_findall = self._re_findall
+        else:
+            re_findall = '|'.join([q.re_pattern.pattern.decode() for q in quantities])
+            if len(quantities) == 1:
+                # necessary to add a dummy variable to make multiple matches
+                re_findall = '%s|(__dummy__)' % re_findall
+            re_findall = re_findall.encode()
+            if self._re_findall is None:
+                self._re_findall = re.compile(re_findall)
 
         # map matches to quantities
         matches = re.findall(re_findall, self.file_mmap)
@@ -530,5 +553,4 @@ class TextParser(FileParser):
                     if quantity.name not in self._results:
                         self._parse_quantity(quantity)
 
-        super().parse()
         return self
